@@ -1,6 +1,7 @@
 from __future__ import annotations
 import asyncio
 import functools
+import logging
 import sys
 import time
 import traceback
@@ -129,6 +130,23 @@ class ProgressHandler:
                 self._error_reporter(message)
 
 
+class InterceptHandler(logging.Handler):
+    def emit(self, record):
+        # Get corresponding Loguru level if it exists
+        try:
+            level = logger.level(record.levelname).name
+        except ValueError:
+            level = record.levelno
+
+        # Find caller from where originated the logged message
+        frame, depth = logging.currentframe(), 2
+        while frame.f_code.co_filename == logging.__file__:
+            frame = frame.f_back
+            depth += 1
+
+        logger.opt(depth=depth, exception=record.exc_info).log(level, record.getMessage())
+
+
 DEFAULT_FORMAT = (
     "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | "
     "<level>{level: <8}</level> | "
@@ -214,6 +232,10 @@ def reset_to_defaults() -> loguru.Logger:
     logger.remove()
     logger.configure(handlers=DEFAULT_HANDLERS)
     DEFAULT_FILTER.level = "INFO"
+
+    # Intercept messages from backoff library
+    logging.getLogger('backoff').addHandler(InterceptHandler())
+
     return logger
 
 def friendly_decorator(f):
